import 'dart:async';
import 'dart:developer';

import 'package:flutter/services.dart';

typedef OfferwallAvailable = void Function(bool isAvailable);
typedef OfferwallOpened = void Function();
typedef OfferwallShowFailed = void Function(int code);
typedef OfferwallAdCredited = void Function(int credits, int totalCredits, bool totalCreditsFlag);
typedef GetOfferwallCreditsFailed = void Function(int code);
typedef OfferwallClosed = void Function();

typedef RewardedVideoAdOpened = void Function();
typedef RewardedVideoAdClosed = void Function();
typedef RewardedVideoAvailabilityChanged = void Function(bool isAvailable);
typedef RewardedVideoAdStarted = void Function();
typedef RewardedVideoAdEnded = void Function();
typedef RewardedVideoAdRewarded = void Function(String rewardName, int rewardAmount);
typedef RewardedVideoAdShowFailed = void Function(int code);
typedef RewardedVideoAdClicked = void Function(String rewardName, int rewardAmount);

typedef InterstitialAdReady = void Function();
typedef InterstitialAdLoadFailed = void Function(int code);
typedef InterstitialAdOpened = void Function();
typedef InterstitialAdClosed = void Function();
typedef InterstitialAdShowSucceeded = void Function();
typedef InterstitialAdShowFailed = void Function(int code);
typedef InterstitialAdClicked = void Function();

class CsIronsourceAd {
  final MethodChannel _channel = const MethodChannel('cs_ironsource_android');
  static final CsIronsourceAd _instance = CsIronsourceAd._internal();
  factory CsIronsourceAd() => _instance;
  CsIronsourceAd._internal() {
    _channel.setMethodCallHandler(_platformCallHandler);
  }

  late OfferwallAvailable _offerwallAvailable;
  late OfferwallOpened _offerwallOpened;
  late OfferwallShowFailed _offerwallShowFailed;
  late OfferwallAdCredited _offerwallAdCredited;
  late GetOfferwallCreditsFailed _getOfferwallCreditsFailed;
  late OfferwallClosed _offerwallClosed;

  late RewardedVideoAdOpened _rewardedVideoAdOpened;
  late RewardedVideoAdClosed _rewardedVideoAdClosed;
  late RewardedVideoAvailabilityChanged _rewardedVideoAvailabilityChanged;
  late RewardedVideoAdStarted _rewardedVideoAdStarted;
  late RewardedVideoAdEnded _rewardedVideoAdEnded;
  late RewardedVideoAdRewarded _rewardedVideoAdRewarded;
  late RewardedVideoAdShowFailed _rewardedVideoAdShowFailed;
  late RewardedVideoAdClicked _rewardedVideoAdClicked;

  late InterstitialAdReady _interstitialAdReady;
  late InterstitialAdLoadFailed _interstitialAdLoadFailed;
  late InterstitialAdOpened _interstitialAdOpened;
  late InterstitialAdClosed _interstitialAdClosed;
  late InterstitialAdShowSucceeded _interstitialAdShowSucceeded;
  late InterstitialAdShowFailed _interstitialAdShowFailed;
  late InterstitialAdClicked _interstitialAdClicked;

  Future<void> initSDK({required String appKey}) async {
    return _channel.invokeMethod("init", <String, dynamic>{
      'appKey': appKey,
    });
  }

  Future<void> setUserId({required String userId}) async {
    return _channel.invokeMethod("setUserId", <String, dynamic>{
      'userId': userId,
    });
  }

  Future<void> showOfferwallAd(String name) {
    return _channel.invokeMethod("showOfferWall", <String, dynamic>{
      'name': name,
    });
  }

  Future<void> showVideoAd(String name) {
    return _channel.invokeMethod("showVideoAd", <String, dynamic>{
      'name': name,
    });
  }

  Future _platformCallHandler(MethodCall call) async {
    switch (call.method) {
      case "offerwallAvailable":
        _offerwallAvailable(call.arguments?[0]);
        log("offerwallAvailable");
        break;
      case "offerwallOpened":
        _offerwallOpened();
        log("offerwallOpened");
        break;
      case "offerwallShowFailed":
        _offerwallShowFailed(call.arguments?[0]);
        log("offerwallShowFailed");
        break;
      case "offerwallAdCredited":
        _offerwallAdCredited(call.arguments?[0], call.arguments?[1], call.arguments?[2]);
        log("offerwallAdCredited");
        break;
      case "getOfferwallCreditsFailed":
        _getOfferwallCreditsFailed(call.arguments);
        log("offerwallAdCredited");
        break;
      case "offerwallClosed":
        _offerwallClosed();
        log("offerwallClosed");
        break;

      case "rewardedVideoAdClicked":
        _rewardedVideoAdClicked(call.arguments?[0], call.arguments?[1]);
        log("rewardedVideoAdClicked");
        break;
      case "rewardedVideoAdShowFailed":
        _rewardedVideoAdShowFailed(call.arguments?[0]);
        log("rewardedVideoAdShowFailed");
        break;
      case "rewardedVideoAdRewarded":
        _rewardedVideoAdRewarded(call.arguments?[0], call.arguments?[1]);
        log("rewardedVideoAdRewarded");
        break;
      case "rewardedVideoAdEnded":
        _rewardedVideoAdEnded();
        log("rewardedVideoAdEnded");
        break;
      case "rewardedVideoAdStarted":
        _rewardedVideoAdStarted();
        log("rewardedVideoAdStarted");
        break;
      case "rewardedVideoAvailabilityChanged":
        _rewardedVideoAvailabilityChanged(call.arguments?[0]);
        log("rewardedVideoAvailabilityChanged");
        break;
      case "rewardedVideoAdClosed":
        _rewardedVideoAdClosed();
        log("rewardedVideoAdClosed");
        break;
      case "rewardedVideoAdOpened":
        _rewardedVideoAdOpened();
        log("rewardedVideoAdOpened");
        break;
      case "interstitialAdClicked":
        _interstitialAdClicked();
        log("interstitialAdClicked");
        break;
      case "interstitialAdShowFailed":
        _interstitialAdShowFailed(call.arguments?[0]);
        log("interstitialAdShowFailed");
        break;
      case "interstitialAdShowSucceeded":
        _interstitialAdShowSucceeded();
        log("interstitialAdShowSucceeded");
        break;
      case "interstitialAdClosed":
        _interstitialAdClosed();
        log("interstitialAdClosed");
        break;
      case "interstitialAdOpened":
        _interstitialAdOpened();
        log("interstitialAdOpened");
        break;
      case "interstitialAdLoadFailed":
        _interstitialAdLoadFailed(call.arguments?[0]);
        log("interstitialAdLoadFailed");
        break;
      case "interstitialAdReady":
        _interstitialAdReady();
        log("interstitialAdReady");
        break;
      default:
        log('Unknown method ${call.method} ');
    }
  }

  void setOfferwallAvailable(OfferwallAvailable offerwallAvailable) => _offerwallAvailable = offerwallAvailable;
  void setOfferwallOpened(OfferwallOpened offerwallOpened) => _offerwallOpened = offerwallOpened;
  void setOfferwallShowFailed(OfferwallShowFailed offerwallShowFailed) => _offerwallShowFailed = offerwallShowFailed;
  void setOfferwallAdCredited(OfferwallAdCredited offerwallAdCredited) => _offerwallAdCredited = offerwallAdCredited;
  void setGetOfferwallCreditsFailed(GetOfferwallCreditsFailed getOfferwallCreditsFailed) => _getOfferwallCreditsFailed = getOfferwallCreditsFailed;
  void setOfferwallClosed(OfferwallClosed offerwallClosed) => _offerwallClosed = offerwallClosed;

  void setRewardedVideoAdOpened(RewardedVideoAdOpened rewardedVideoAdOpened) => _rewardedVideoAdOpened = rewardedVideoAdOpened;
  void setRewardedVideoAdClosed(RewardedVideoAdClosed rewardedVideoAdClosed) => _rewardedVideoAdClosed = rewardedVideoAdClosed;
  void setRewardedVideoAvailabilityChanged(RewardedVideoAvailabilityChanged rewardedVideoAvailabilityChanged) => _rewardedVideoAvailabilityChanged = rewardedVideoAvailabilityChanged;
  void setRewardedVideoAdStarted(RewardedVideoAdStarted rewardedVideoAdStarted) => _rewardedVideoAdStarted = rewardedVideoAdStarted;
  void setRewardedVideoAdEnded(RewardedVideoAdEnded rewardedVideoAdEnded) => _rewardedVideoAdEnded = rewardedVideoAdEnded;
  void setRewardedVideoAdRewarded(RewardedVideoAdRewarded rewardedVideoAdRewarded) => _rewardedVideoAdRewarded = rewardedVideoAdRewarded;
  void setRewardedVideoAdShowFailed(RewardedVideoAdShowFailed rewardedVideoAdShowFailed) => _rewardedVideoAdShowFailed = rewardedVideoAdShowFailed;
  void setRewardedVideoAdClicked(RewardedVideoAdClicked rewardedVideoAdClicked) => _rewardedVideoAdClicked = rewardedVideoAdClicked;

  void setInterstitialAdReady(InterstitialAdReady interstitialAdReady) => _interstitialAdReady = interstitialAdReady;
  void setInterstitialAdLoadFailed(InterstitialAdLoadFailed interstitialAdLoadFailed) => _interstitialAdLoadFailed = interstitialAdLoadFailed;
  void setInterstitialAdOpened(InterstitialAdOpened interstitialAdOpened) => _interstitialAdOpened = interstitialAdOpened;
  void setInterstitialAdClosed(InterstitialAdClosed interstitialAdClosed) => _interstitialAdClosed = interstitialAdClosed;
  void setInterstitialAdShowSucceeded(InterstitialAdShowSucceeded interstitialAdShowSucceeded) => _interstitialAdShowSucceeded = interstitialAdShowSucceeded;
  void setInterstitialAdShowFailed(InterstitialAdShowFailed interstitialAdShowFailed) => _interstitialAdShowFailed = interstitialAdShowFailed;
  void setInterstitialAdClicked(InterstitialAdClicked interstitialAdClicked) => _interstitialAdClicked = interstitialAdClicked;
}
